@@ -18,12 +18,12 @@ ACTION="$1"
 update_packages() {
     pip install --upgrade pip
     pip install --upgrade setuptools
-    pip install -r "$BASE_DIR/requirements.txt"
+    pip install -Ur "$BASE_DIR/requirements.txt"
 }
 
 update_dev_packages() {
     update_packages
-    pip install -r "$BASE_DIR/requirements-dev.txt"
+    pip install -Ur "$BASE_DIR/requirements-dev.txt"
 }
 
 install_geckodriver() {
@@ -35,7 +35,7 @@ install_geckodriver() {
     if [ -z "$NOTFOUND" ]; then
         return
     fi
-    GECKODRIVER_VERSION="v0.19.1"
+    GECKODRIVER_VERSION="v0.24.0"
     PLATFORM="`python -c "import six; import platform; six.print_(platform.system().lower(), platform.architecture()[0])"`"
     case "$PLATFORM" in
         "linux 32bit" | "linux2 32bit") ARCH="linux32";;
@@ -70,6 +70,11 @@ locales() {
     pybabel compile -d "$SEARX_DIR/translations"
 }
 
+update_useragents() {
+    echo '[!] Updating user agent versions'
+    python utils/fetch_firefox_version.py
+}
+
 pep8_check() {
     echo '[!] Running pep8 check'
     # ignored rules:
@@ -152,12 +157,82 @@ styles() {
 }
 
 grunt_build() {
+    npm_path_setup
+
     echo '[!] Grunt build : oscar theme'
     grunt --gruntfile "$SEARX_DIR/static/themes/oscar/gruntfile.js"
     echo '[!] Grunt build : simple theme'
     grunt --gruntfile "$SEARX_DIR/static/themes/simple/gruntfile.js"
 }
 
+docker_build() {
+    # Check if it is a git repository
+    if [ ! -d .git ]; then
+	echo "This is not Git repository"
+	exit 1
+    fi
+
+    if [ ! -x "$(which git)" ]; then
+	echo "git is not installed"
+	exit 1
+    fi
+
+    if [ ! git remote get-url origin 2> /dev/null ]; then
+	echo "there is no remote origin"
+	exit 1
+    fi
+
+    # This is a git repository
+
+    # "git describe" to get the Docker version (for example : v0.15.0-89-g0585788e)
+    # awk to remove the "v" and the "g"
+    SEARX_GIT_VERSION=$(git describe --match "v[0-9]*\.[0-9]*\.[0-9]*" HEAD 2>/dev/null | awk -F'-' '{OFS="-"; $1=substr($1, 2); $3=substr($3, 2); print}')
+
+    # add the suffix "-dirty" if the repository has uncommited change
+    git update-index -q --refresh
+    if [ ! -z "$(git diff-index --name-only HEAD --)" ]; then
+	SEARX_GIT_VERSION="${SEARX_GIT_VERSION}-dirty"
+    fi
+
+    # Get the last git commit id, will be added to the Searx version (see Dockerfile)
+    VERSION_GITCOMMIT=$(echo $SEARX_GIT_VERSION | cut -d- -f2-4)
+    echo "Last commit : $VERSION_GITCOMMIT"
+
+    # Check consistency between the git tag and the searx/version.py file
+    # /!\ HACK : parse Python file with bash /!\
+    # otherwise it is not possible build the docker image without all Python dependencies ( version.py loads __init__.py )
+    # SEARX_PYTHON_VERSION=$(python -c "import six; import searx.version; six.print_(searx.version.VERSION_STRING)")
+    SEARX_PYTHON_VERSION=$(cat searx/version.py | grep "\(VERSION_MAJOR\|VERSION_MINOR\|VERSION_BUILD\) =" | cut -d\= -f2 | sed -e 's/^[[:space:]]*//' | paste -sd "." -)
+    if [ $(echo "$SEARX_GIT_VERSION" | cut -d- -f1) != "$SEARX_PYTHON_VERSION" ]; then
+	echo "Inconsistency between the last git tag and the searx/version.py file"
+	echo "git tag:          $SEARX_GIT_VERSION"
+	echo "searx/version.py: $SEARX_PYTHON_VERSION"
+	exit 1
+    fi
+
+    # define the docker image name
+    # /!\ HACK to get the user name /!\
+    GITHUB_USER=$(git remote get-url origin | sed 's/.*github\.com\/\([^\/]*\).*/\1/')
+    SEARX_IMAGE_NAME="${GITHUB_USER:-searx}/searx"
+
+    # build Docker image
+    echo "Building image ${SEARX_IMAGE_NAME}:${SEARX_GIT_VERSION}"
+    sudo docker build \
+         --build-arg SEARX_GIT_VERSION="${SEARX_GIT_VERSION}" \
+         --build-arg VERSION_GITCOMMIT="${VERSION_GITCOMMIT}" \
+         --build-arg LABEL_DATE=$(date -u +"%Y-%m-%dT%H:%M:%SZ") \
+         --build-arg LABEL_VCS_REF=$(git rev-parse HEAD) \
+         --build-arg LABEL_VCS_URL=$(git remote get-url origin) \
+	 --build-arg TIMESTAMP_SETTINGS=$(git log -1 --format="%cd" --date=unix -- searx/settings.yml) \
+	 --build-arg TIMESTAMP_UWSGI=$(git log -1 --format="%cd" --date=unix -- dockerfiles/uwsgi.ini) \
+         -t ${SEARX_IMAGE_NAME}:latest -t ${SEARX_IMAGE_NAME}:${SEARX_GIT_VERSION} .
+
+    if [ "$1" = "push" ]; then
+	sudo docker push ${SEARX_IMAGE_NAME}:latest
+	sudo docker push ${SEARX_IMAGE_NAME}:${SEARX_GIT_VERSION}
+    fi
+}
+
 #
 # Help
 #
@@ -175,13 +250,15 @@ Commands
     update_packages      - Check & update production dependency changes
     update_dev_packages  - Check & update development and production dependency changes
     install_geckodriver  - Download & install geckodriver if not already installed (required for robot_tests)
-    npm_packages         - Download & install npm dependencies (source manage.sh to update the PATH)
+    npm_packages         - Download & install npm dependencies
+    update_useragents    - Update useragents.json with the most recent versions of Firefox
 
     Build
     -----
     locales              - Compile locales
     styles               - Build less files
     grunt_build          - Build files for themes
+    docker_build         - Build Docker image
 
     Tests
     -----
